<?php
define('EUROSMS_MAX_LANGUAGES', 3);

class AdminEuroSmsController extends ModuleAdminController {
  protected static $allowedTokens = NULL;

  function __construct() {
    $this->bootstrap = true;
    $this->lang = true;
    $this->display = 'view';
    $this->context = Context::getContext();
    $this->base_tpl_view = 'content.tpl';

    require_once (_PS_MODULE_DIR_ . 'eurosms/classes/model/EuroSmsNotification.php');
    parent::__construct();

    $link = new Link();

    $languages = array();
    $langs = Language::getLanguages(true);
    $cnt = 0;
    foreach ($langs as $lang) {
      $cnt++;
      $languages[$lang['iso_code']] = $lang;
      if ($cnt >= EUROSMS_MAX_LANGUAGES) {
        break;
      }
    }

    $shops = array();
    $shps = Shop::getShops(true);
    foreach ($shps as $shp) {
      $shop = Db::getInstance()->ExecuteS('SELECT `name`, `value` FROM `' . _DB_PREFIX_ . 'configuration` WHERE name like \'PS_SHOP%\' AND (id_shop IS NULL OR id_shop=' . $shp['id_shop'] . ') ORDER BY id_shop ASC');
      $shop_conf = array(
        'id_shop' => $shp['id_shop'],
        'id' => $shp['id_shop']
      );
      foreach ($shop as $shop_entry) {
        $shop_entry_name = strtolower(str_replace('PS_SHOP_', '', $shop_entry['name']));
        $shop_conf[$shop_entry_name] = $shop_entry['value'];
      }

      $shops[$shp['name']] = $shop_conf;
    }

    $countries = array();
    $ctries = Country::getCountries(Context::getContext()->language->id);
    foreach ($ctries as $ctry) {
      $countries[$ctry['id_country']] = array(
        'code' => $ctry['iso_code'],
        'name' => Country::getNameById(Context::getContext()->language->id, $ctry['id_country']) ,
        'id_country' => $ctry['id_country'],
        'id' => $ctry['id_country']
      );
    }

    $tokens = EuroSmsNotification::getTokens(static::$allowedTokens);

    $this->tpl_view_vars = array(
      'url' => __PS_BASE_URI__ . basename(_PS_ADMIN_DIR_) . '/' . $link->getAdminLink(str_replace('Controller', '', get_class($this))) ,
      'baseUrl' => __PS_BASE_URI__ . basename(_PS_ADMIN_DIR_) ,
      'link' => $link,

      'languages' => $languages,
      'shops' => $shops,
      'countries' => $countries,

      'tokens' => $tokens,
      'tz' => date_default_timezone_get() ,
      'request' => $_REQUEST
    );
  }

  public function setMedia() {
    parent::setMedia();
    //$this->addJqueryUi('ui.widget');
    
  }

  public function initContent() {
    $ajaxCmd = Tools::getValue('ajax');
    if ($ajaxCmd) {
      $ajaxResponse = $this->esProcessAjax($ajaxCmd);
      die(Tools::jsonEncode($ajaxResponse));
    }
    else {
      parent::initContent();
    }
  }

  public function getContent() {
    return $this->renderView();
  }

  public function renderView() {
    return parent::renderView();
  }

  protected function esProcessAjax($ajaxCmd) {
    // no code
    
  }

  public function esLoadModel($id = NULL) {
    $className = $this->className;
    $model = new $className($id);
    return $model;
  }

  public function esSetViewVars($model) {
    foreach (get_object_vars($model) as $var => $val) {
      $this->tpl_view_vars[$var] = $model->$var;
    }

    $this->tpl_view_vars['model'] = $model;
  }

  protected function esSaveModel($model, $confirmMessage = false) {
    $result = false;
    try {
      $result = $model->esSave();
      if ($result && $confirmMessage) {
        $this->confirmations[] = $confirmMessage;
      }
    }
    catch(Exception $e) {
      $this->errors[] = $this->l($e->getMessage());
    }

    return $result;
  }

  protected function esDeleteModel($model, $confirmMessage = false) {
    $result = false;
    try {
      $result = $model->esDelete();
      if ($result && $confirmMessage) {
        $this->confirmations[] = $confirmMessage;
      }
    }
    catch(Exception $e) {
      $this->errors[] = $this->l($e->getMessage());
    }

    return $result;
  }
}
