<?php
class EuroSmsModel extends ObjectModel {

  function __construct($id = null) {
    parent::__construct($id);
  }

  protected function makeNullValues() {
    foreach (get_object_vars($this) as $var => $val) {
      $def = @static ::$definition['fields'][$var];
      if ($def && !empty($def['allow_null']) && ($val === '')) {
        $this->$var = null;
      }
    }
  }

  private function __esPrepareCall() {
    // TODO[clean]: Don't know why, $this->def is not set during es-calls
    $this->def = ObjectModel::getDefinition(get_class($this));
  }

  protected function protectValues() {
    // not required - protection (formatting) is done in ObjectModel class through getFields() -> formatFields() -> formatValue() methods
  }

  public function update($null_values = false) {
    $this->makeNullValues();
    $this->protectValues();
    return parent::update($null_values);
  }

  public function add($autodate = true, $null_values = false) {
    $this->makeNullValues();
    $this->protectValues();
    return parent::add($autodate, $null_values);
  }

  public function esSetFromRequest() {
    $this->__esPrepareCall();
    $data = array();
    foreach (get_object_vars($this) as $var => $val) {
      $value = Tools::getValue($var, null);
      if (isset($value)) {
        $data[$var] = $value;
      }
    }

    $this->esSet($data);
  }

  public function esSet($data) {
    $this->__esPrepareCall();
    $primary = static ::$definition['primary'];
    foreach (get_object_vars($this) as $var => $val) {
      if (!isset(static ::$definition['fields'][$var])) {
        continue;
      }

      $value = @$data[$var];
      if (in_array($var, array(
        $primary,
        'id'
      )) && !isset($value)) {
        continue;
      }

      $def = static ::$definition['fields'][$var];
      if (!isset($value) && $def['required']) {
        switch ($def['type']) {
          case static ::TYPE_STRING:
            $value = '';
          break;
          case static ::TYPE_FLOAT:
          case static ::TYPE_INT:
            $value = 0;
          break;
          case static ::TYPE_BOOL:
            $value = false;
          break;
        }
      }
      $this->$var = $value;
    }
  }

  public static function esGetMsgError() {
    $db = Db::getInstance();
    return $db->getMsgError() . ' [' . $db->getNumberError() . ']';
  }

  public function esSave() {
    $this->__esPrepareCall();
    $result = $this->save(true);

    if (!$result) {
      $db = Db::getInstance();
      throw new Exception(static ::esGetMsgError());
    }

    return $result;
  }

  public function esDelete() {
    $this->__esPrepareCall();
    $result = $this->delete();
    if (!$result) {
      $db = Db::getInstance();
      throw new Exception(static ::esGetMsgError());
    }

    return $result;
  }

  public static function l($string) {
    return Translate::getModuleTranslation('eurosms', $string, 'eurosms');
  }
}