<?php
require_once (__DIR__ . '/../EuroSmsModel.php');
require_once ('EuroSmsRecipientFilter.php');
class EuroSmsConfiguration extends EuroSmsModel {

  const CONFIGURATION_ID = 1;

  public $id_euro_sms_configuration = null;
  public $email = '';
  public $personal = false;
  public $company_name = '';
  public $company_id = null;
  public $company_vat = null;
  public $contact_first_name = null;
  public $contact_last_name = null;
  public $address_street = null;
  public $address_city = null;
  public $address_zip = null;
  public $address_id_country = null;
  public $user_name = null;
  public $user_password = null;
  public $administrators = '{}';
  //public $phone_invoice = false;
  public $credit_alert = null;
  public $cron_key = '';
  public $simulation = false;
  public $delivery_reports = true;

  public static $definition = array(
    'table' => 'euro_sms_configuration',
    'primary' => 'id_euro_sms_configuration',

    //'multilang' => true,
    'multilang_shop' => true,
    'multishop' => true,

    'associations' => array(
      'address_country' => array(
        'type' => self::HAS_ONE,
        'field' => 'address_id_country',
        'object' => 'Country'
      ) ,
    ) ,
    'fields' => array(
      'id_euro_sms_configuration' => array(
        'type' => self::TYPE_INT,
        'validate' => 'isUnsignedId',
        'copy_post' => false
      ) ,
      'email' => array(
        'type' => self::TYPE_STRING,
        'validate' => 'isEmail',
        'required' => true
      ) ,
      'personal' => array(
        'type' => self::TYPE_BOOL,
        'required' => true
      ) ,
      'company_name' => array(
        'type' => self::TYPE_STRING,
        'required' => true
      ) ,
      'company_id' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      'company_vat' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      'contact_first_name' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      'contact_last_name' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      'address_street' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      'address_city' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      'address_zip' => array(
        'type' => self::TYPE_STRING,
        'validate' => 'isPostCode',
        'required' => false
      ) ,
      'address_id_country' => array(
        'type' => self::TYPE_INT,
        'validate' => 'isNullOrUnsignedId',
        'allow_null' => true
      ) ,
      'user_name' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      'user_password' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      'administrators' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      /*'phone_invoice' => array(
        'type' => self::TYPE_BOOL,
        'required' => true
      ) ,*/
      'credit_alert' => array(
        'type' => self::TYPE_STRING,
        'validate' => 'isEmail',
        'required' => false
      ) ,
      'cron_key' => array(
        'type' => self::TYPE_STRING,
        'required' => true,
        'copy_post' => false
      ) ,
      'simulation' => array(
        'type' => self::TYPE_BOOL,
        'required' => true
      ) ,
      'delivery_reports' => array(
        'type' => self::TYPE_BOOL,
        'required' => true
      ) ,
    ) ,
  );

  function __construct($id = self::CONFIGURATION_ID) {
    if (!isset($id)) {
      $id = self::CONFIGURATION_ID;
    }
    parent::__construct($id);
    if (!$this->cron_key) {
      $this->cron_key = md5(time());
    }

    /*if ($this->administrators) {
      $this->administrators = json_decode($this->administrators);
    }*/
  }

  /*public function save($nullValues = false, $autodate = true) {
  }*/

  public function getAdministrators() {
    $admin_obj = array();
    if ($this->administrators) {
      $admin_obj = json_decode($this->administrators);
    }

    foreach($admin_obj as &$admin_entry) {
      $phone = trim($admin_entry->phone);
      if ($phone) {
        $admin_entry->phone = EuroSmsRecipientFilter::formatPhoneNumber($phone);
      }
      else {
        $admin_entry = null;
      }
    }

    $admin_obj = array_filter($admin_obj);

    return $admin_obj;
  }

  public function sendCreditAlert($variables = array(), $subject = 'Low credit', $template = 'low_credit') {
    Logger::addLog('Low credit alert from provider: ' . @$variables['{provider}'], 3);

    if ($this->credit_alert) {
      $shop_email = strval(Configuration::get('PS_SHOP_EMAIL'));
      $shop_name = strval(Configuration::get('PS_SHOP_NAME'));
      return Mail::Send(Context::getContext()->language->id, $template, Mail::l($subject, Context::getContext()->language->id) , $variables, $this->credit_alert, '', $shop_email, $shop_name, array() /*$filesAtt*/, NULL, _PS_MODULE_DIR_ . '/eurosms/mails/');
    }
  }
}
