<?php
require_once (__DIR__ . '/../EuroSmsModel.php');
require_once ('EuroSmsConfiguration.php');
require_once ('EuroSmsRecipientFilter.php');
require_once (_PS_MODULE_DIR_ . '/eurosms/classes/api/manager.php');
class EuroSmsHistory extends EuroSmsModel {

  const PROVIDER_DEFAULT = 'eurosms';
  const PROVIDER_MOCK = 'mock';

  const TYPE_SENDING = 'sending';
  const TYPE_MARKETING = 'marketing';
  const TYPE_NOTIFICATION = 'notification';

  const STATUS_SENT = EuroSmsManager::STATUS_SENT;
  const STATUS_DELIVERED = EuroSmsManager::STATUS_DELIVERED;
  const STATUS_PENDING = EuroSmsManager::STATUS_PENDING;
  const STATUS_UNDELIVERED = EuroSmsManager::STATUS_UNDELIVERED;

  const DEFAULT_MESSAGE_LENGTH = 160;
  const DEFAULT_NEXT_MESSAGE_LENGTH = 153;
  const UNICODE_MESSAGE_LENGTH = 70;
  const UNICODE_NEXT_MESSAGE_LENGTH = 67;

  public $id_euro_sms_history = null;
  public $sender = null;
  public $recipient = '';
  public $timestamp = null;
  public $delivery_timestamp = null;
  public $type = 'sending';
  public $simulation = false;
  public $status = 'pending';
  public $error = null;
  public $detail = null;
  public $text = '';
  public $price = 0;
  public $unicode = false;
  public $sms_id = null;
  public $group_id = null;
  public $sms_count = 1;

  public static $definition = array(
    'table' => 'euro_sms_history',
    'primary' => 'id_euro_sms_history',

    //'multilang' => true,
    'multilang_shop' => true,
    'multishop' => true,

    'associations' => array() ,
    'fields' => array(
      'id_euro_sms_history' => array(
        'type' => self::TYPE_INT,
        'validate' => 'isUnsignedId',
        'copy_post' => false
      ) ,
      'sender' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      'recipient' => array(
        'type' => self::TYPE_STRING,
        'required' => true
      ) ,
      'timestamp' => array(
        'type' => self::TYPE_DATE,
        'validate' => 'isDate',
        'required' => true,
        'copy_post' => false
      ) ,
      'delivery_timestamp' => array(
        'type' => self::TYPE_DATE,
        'validate' => 'isDate',
        'required' => false
      ) ,
      'type' => array(
        'type' => self::TYPE_STRING,
        'required' => true
      ) ,
      'simulation' => array(
        'type' => self::TYPE_BOOL,
        'required' => true
      ) ,
      'status' => array(
        'type' => self::TYPE_STRING,
        'required' => true
      ) ,
      'error' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      'detail' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      'text' => array(
        'type' => self::TYPE_STRING,
        'required' => true
      ) ,
      'price' => array(
        'type' => self::TYPE_FLOAT,
        'validate' => 'isUnsignedFloat',
        'required' => true
      ) ,
      'unicode' => array(
        'type' => self::TYPE_BOOL,
        'required' => true
      ) ,
      'sms_id' => array(
        'type' => self::TYPE_STRING,
        'required' => false,
        'allow_null' => true
      ) ,
      'group_id' => array(
        'type' => self::TYPE_STRING,
        'required' => false,
        'allow_null' => true
      ) ,
      'sms_count' => array(
        'type' => self::TYPE_INT,
        'validate' => 'isUnsignedInt',
        'required' => true
      ) ,
    ) ,
  );

  function __construct($id = NULL) {
    parent::__construct($id);
    if (!$this->timestamp) {
      $this->timestamp = date('Y-m-d H:i:s');
    }
  }

  protected static function getConfiguration() {
    static $config = null;
    if (!$config) {
      $config = new EuroSmsConfiguration();
    }

    return $config;
  }

  public function save($nullValues = false, $autodate = true) {
    $this->prepare();
    return parent::save($nullValues, $autodate);
  }

  protected function prepare() {
    if (!$this->sms_count) {
      $msgLen = $this->unicode ? static ::UNICODE_MESSAGE_LENGTH : static ::DEFAULT_MESSAGE_LENGTH;
      $msgNextLen = $this->unicode ? static ::UNICODE_NEXT_MESSAGE_LENGTH : static ::UNICODE_MESSAGE_LENGTH;
      $textLen = strlen($this->text);

      $this->sms_count = ceil(max(0, $textLen - $msgLen) / $msgNextLen) + 1;
    }

    if (!strtotime($this->delivery_timestamp)) {
      $this->delivery_timestamp = null;
    }
  }

  protected static function prepareMessage($history) {
    if (!$history->id) {
      $history->status = static ::STATUS_UNDELIVERED;
      $history->esSave();
    }

    $history->id_euro_sms_history = $history->id;
    $sender = trim($history->sender);
    if (!$sender) {
      $history->sender = $sender = EuroSmsRecipientFilter::formatPhoneNumber(Configuration::get('PS_SHOP_PHONE'));
    }

    $msgEntry = (object)array(
      'sender' => $sender,
      'recipient' => $history->recipient,
      'unicode' => $history->unicode,
      'text' => $history->text,
      'id' => $history->id,
      'timestamp' => $history->timestamp,
      'sms_id' => $history->sms_id,
      'group_id' => $history->group_id,
      'price' => $history->price
    );

    return $msgEntry;
  }

  protected static function getApiManager() {
    $config = static ::getConfiguration();
    // TODO[hardcoded]: fixed provider
    $mgr = new EuroSmsManager(array(
      static ::PROVIDER_DEFAULT => (object)array(
        'authenticate' => (object)array(
          'api_id' => $config->user_name,
          'api_key' => $config->user_password
        )
      )
    ));

    return $mgr;
  }

  public static function sendMany($histories = array()) {
    $msgList = array();
    foreach ($histories as & $history) {
      if (!isset($history->status) && $history->text) {
        $msg = static ::prepareMessage($history);
        $msg->history = & $history;
        $msgList[$history->id] = $msg;
      }
    }

    unset($history);

    if ($msgList) {
      // TODO[hardcoded]: fixed provider
      $mgr = static ::getApiManager();
      $config = static ::getConfiguration();

      $providerName = $config->simulation ? static ::PROVIDER_MOCK : static ::PROVIDER_DEFAULT;
      if ($response = $mgr->sendMessage($msgList, NULL, array(
        $providerName
      ))) {

        if (isset($response[$providerName]) && ($response[$providerName] === FALSE)) {
          throw new Exception('Unknown error, please, verify module configuration or contact module supplier');
        }

        if (isset($response[$providerName]['error'])) {
          if (in_array('Low credit', $response[$providerName]['error'])) {
            $config->sendCreditAlert(array(
              '{provider}' => strtoupper($providerName)
            ));
          }

          //var_dump($response['error']); die();

          $error_str = implode(" | ", array_map(array(
            'EuroSmsModel',
            'l'
          ) , $response[$providerName]['error']));

          foreach ($msgList as $msg) {
            $msg->history->error = $error_str;
            $msg->history->esSave();
          }

          throw new Exception($error_str);
        }

        foreach ($response as $resp_msg) {
          $history = $msgList[$resp_msg->id]->history;
          if (isset($resp_msg->error)) {
            $history->status = static ::STATUS_UNDELIVERED;
            $history->error = $resp_msg->error;
          }
          else {
            $history->status = static ::STATUS_SENT;
            if (isset($resp_msg->sms_count)) {
              $history->sms_count = $resp_msg->sms_count;
            }
            else {
              $history->sms_count = 0;
            }

            $history->sms_id = $resp_msg->sms_id;
            $history->group_id = $resp_msg->group_id;
          }

          $history->esSave();
        }

        return true;
      }
      else {
        return false;
      }
    }

    return true;
  }

  public static function checkStatuses($histories = array()) {
    $msgList = array();
    foreach ($histories as & $history) {
      if (isset($history->status)) {
        $msg = static ::prepareMessage($history);
        $msg->history = & $history;
        $msgList[$history->id] = $msg;
      }
    }

    unset($history);

    $mgr = static ::getApiManager();
    $config = static ::getConfiguration();

    $providerName = $config->simulation ? static ::PROVIDER_MOCK : static ::PROVIDER_DEFAULT;
    if ($response = $mgr->getMessageStatus($msgList, NULL, array(
      $providerName
    ))) {
     
      if (isset($response[$providerName]) && ($response[$providerName] === FALSE)) {
        throw new Exception('Unknown error, please, verify module configuration or contact module supplier');
       }

      if (isset($response[$providerName]['error'])) {
        throw new Exception(implode(" | ", array_map(array(
          'EuroSmsModel',
          'l'
        ) , $response[$providerName]['error'])));
      }

      foreach ($response as $resp_msg) {
        $history = $msgList[$resp_msg->id]->history;
        $save = false;
        if (isset($resp_msg->price)) {
          $save = true;
          $history->price = $resp_msg->price;
        }

        if (isset($resp_msg->delivery_timestamp)) {
          $save = true;
          $history->delivery_timestamp = date('Y-m-d H:i:s', $resp_msg->delivery_timestamp);
        }

        if (isset($resp_msg->status)) {
          $save = true;
          $history->status = $resp_msg->status;
        }

        if (isset($resp_msg->error)) {
          $save = true;
          $history->error = $resp_msg->error;
        }

        if ($save) {
          $history->esSave();
        }
      }
    }
    else {
      return false;
    }

    return true;
  }

  public function send() {
    if (!isset($this->status)) {
      $msg = static ::prepareMessage($this);
      // TODO[hardcoded]: fixed provider
      $mgr = static ::getApiManager();

      $config = static ::getConfiguration();
      if ($response = $mgr->sendMessage(array(
        $msg
      ) , NULL, array(
        $config->simulation ? static ::PROVIDER_MOCK : static ::PROVIDER_DEFAULT
      ))) {
        if (isset($response['error'])) {
          if (in_array('Low credit', $response['error'])) {
            $config->sendCreditAlert(array(
              '{provider}' => strtoupper($config->simulation ? static ::PROVIDER_MOCK : static ::PROVIDER_DEFAULT)
            ));
          }

          $error_str = implode(" | ", array_map(array(
            'EuroSmsModel',
            'l'
          ) , $response['error']));

          $this->error = $error_str;
          $this->esSave();
          throw new Exception($error_str);
        }
        else {
          $msg = reset($response);
          if ($msg->error) {
            $this->status = static ::STATUS_UNDELIVERED;
            $this->error = $msg->error;
            $this->esSave();
            throw new Exception($msg->error . ': ' . $msg->rcpt);
          }
          else {
            $this->status = static ::STATUS_SENT;
            $this->sms_id = $msg->sms_id;
            $this->group_id = $msg->group_id;
            $this->esSave();
          }

          return true;
        }
      }
      else {
        return false;
      }
    }

    return true;
  }

  public function checkStatus() {
    return static ::checkStatuses(array(
      $this
    ));
  }

  public static function getHistory($filter = array() , $per_page = 0, $page = 1, &$total = NULL, $fields = array()) {
    $cond = '1';

    if (!empty($filter['sender'])) {
      $cond .= ' AND h.`sender` LIKE \'' . pSQL($filter['sender']) . '\'';
    }

    if (!empty($filter['recipient'])) {
      $cond .= ' AND h.`recipient` LIKE \'' . pSQL($filter['recipient']) . '\'';
    }

    if (!empty($filter['text'])) {
      $cond .= ' AND h.`text` LIKE \'%' . pSQL($filter['text']) . '%\'';
    }

    if (!empty($filter['type'])) {
      $filter['type'] = array_map('pSQL', (array)$filter['type']);
      $cond .= ' AND h.`type` IN (\'' . implode("','", array_map('pSQL', $filter['type'])) . '\')';
    }

    if (!empty($filter['status'])) {
      $filter['status'] = array_map('pSQL', (array)$filter['status']);
      $cond .= ' AND h.`status` IN (\'' . implode("','", array_map('pSQL', $filter['status'])) . '\')';
    }

    if (isset($filter['simulation'])) {
      $cond .= ' AND h.`simulation` = ' . (int)$filter['simulation'];
    }

    if (!empty($filter['timestamp']['from'])) {
      $cond .= ' AND h.`timestamp` >= \'' . pSQL($filter['timestamp']['from']) . '\'';
    }

    if (!empty($filter['timestamp']['to'])) {
      $cond .= ' AND h.`timestamp` <= \'' . pSQL($filter['timestamp']['to']) . '\'';
    }

    if ($per_page > 0) {
      $page = (int)$page;
      $per_page = (int)$per_page;
      $limit = ' LIMIT ' . ($page - 1) * $per_page . ', ' . $per_page;
    }
    else {
      $limit = '';
    }

    //var_dump($cond); die();

    $db = Db::getInstance();
    if (isset($total)) {
      $result = $db->executeS('SELECT COUNT(*) AS cnt FROM ' . _DB_PREFIX_ . 'euro_sms_history h WHERE ' . $cond);
      if ($result === false) {
        throw new Exception(static ::esGetMsgError());
      }

      $total = $result[0]['cnt'];
    }

    if (!$fields) {
      $fields = array(
        'h.*',
        'IF (h.timestamp > NOW() AND h.status = \'sent\', 1, 0) AS `later`'
      );
    }

    $result = $db->executeS('SELECT ' . implode(',', $fields) . ' FROM ' . _DB_PREFIX_ . 'euro_sms_history h WHERE ' . $cond . ' ORDER BY h.timestamp DESC ' . $limit);
    if ($result === false) {
      throw new Exception(static ::esGetMsgError());
    }

    return $result;
  }

  public static function loadHistory($filter = array()) {
    $histories = static::getHistory($filter);
    $hist_objs = array();
    foreach($histories as $hist) {
      $hist_obj = new EuroSmsHistory($hist['id_euro_sms_history']);
      $hist_objs[] = $hist_obj;
    }

    return $hist_objs;
  }

  public static function deleteAll() {
    $db = Db::getInstance();
    return $db->execute('DELETE FROM ' . _DB_PREFIX_ . 'euro_sms_history') && $db->execute('DELETE n,s FROM ' . _DB_PREFIX_ . 'euro_sms_notification n INNER JOIN ' . _DB_PREFIX_ . 'euro_sms_sending s ON (n.name = s.notification_name) WHERE s.template IS NULL');
  }
}