<?php
require_once (__DIR__ . '/../EuroSmsModel.php');
require_once ('EuroSmsConfiguration.php');
require_once ('EuroSmsHistory.php');
class EuroSmsNotification extends EuroSmsModel {
  const DEFAULT_PREFIX = '@';
  const DEFAULT_TOKEN_VALUE = '-';

  public static $tokens = array(
    'shop_name',
    'shop_domain',
    'shop_email',
    'shop_phone',
    'customer_firstname',
    'customer_lastname',
    'customer_email',
    'customer_phone',
    'order_shipping_address',
    'order_shipping_city',
    'order_shipping_postcode',
    'order_shipping_country',
    'order_billing_address',
    'order_billing_city',
    'order_billing_postcode',
    'order_billing_country',
    'order_id',
    'order_reference',
    'order_total',
    'order_tracking'
  );

  public $id_euro_sms_notification = null;
  public $name = '';
  public $sender = null;
  public $text = '';
  public $active = 0;
  public $unicode = '';

  private $config;

  public static $definition = array(
    'table' => 'euro_sms_notification',
    'primary' => 'id_euro_sms_notification',

    //'multilang' => true,
    'multilang_shop' => true,
    'multishop' => true,

    'associations' => array() ,
    'fields' => array(
      'id_euro_sms_notification' => array(
        'type' => self::TYPE_INT,
        'validate' => 'isUnsignedId',
        'copy_post' => false
      ) ,
      'name' => array(
        'type' => self::TYPE_STRING,
        'required' => true
      ) ,
      'sender' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      'text' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      'active' => array(
        'type' => self::TYPE_BOOL,
        'required' => true
      ) ,
      'unicode' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
    ) ,
  );

  public static function decodeLanguages($value) {
    return @json_decode($value, true);
  }

  public static function encodeLanguages($value) {
    return is_scalar($value) ? $value : json_encode($value);
  }

  function __construct($id = NULL) {
    parent::__construct($id);
    $this->config = new EuroSmsConfiguration();
    $this->sender = static ::decodeLanguages($this->sender);
    $this->text = static ::decodeLanguages($this->text);
    $this->unicode = static ::decodeLanguages($this->unicode);
  }

  public static function loadByName($name, $active = NULL) {
    $result = Db::getInstance()->executeS('SELECT n.* FROM ' . _DB_PREFIX_ . 'euro_sms_notification n WHERE ' . (isset($active) ? ('n.active = ' . ((bool)$active ? '1' : '0') . ' AND ') : '') . 'n.name = \'' . pSQL($name) . '\'');
    if (!$result) {
      return false;
    }

    $entry = reset($result);
    return new EuroSmsNotification($entry['id_euro_sms_notification']);
  }

  public function send($recipients, $options = array()) {
    $history = array();

    $recipients = array_unique($recipients);
    foreach ($recipients as $key => $recipient) {
      $hist = new EuroSmsHistory();

      $customer = EuroSmsRecipientFilter::getRecipientByPhone($recipient);
      if ($customer) {
        $customer_obj = new Customer($customer['id_customer']);
        $lang_id = $customer_obj->id_lang;
      }
      else {
        $lang_id = 0;
      }

      if (!$lang_id) {
        $lang_id = Context::getContext()->language->id;
      }

      $lang = new Language($lang_id);

      $hist->text = @$this->text[$lang->iso_code];
      if (!$hist->text) {
        $hist->text = reset($this->text);
      }

      if (isset($options['params'][$key])) {
        $params = $options['params'][$key];
      }
      else {
        $params = null;
      }

      $hist->text = static ::replaceTokens($hist->text, $params);

      $hist->unicode = (boolean)@$this->unicode[$lang->iso_code];

      if ($this->sender) {
        $hist->sender = @$this->sender[$lang->iso_code];
        if (!$hist->sender) {
          $hist->sender = reset($this->sender);
        }
      }

      $hist->recipient = $recipient;

      if (isset($options['time']) && strtotime($options['time'])) {
        $hist->timestamp = $options['time'];
      }

      if (isset($options['type'])) {
        $hist->type = $options['type'];
      }
      else {
        $hist->type = EuroSmsHistory::TYPE_NOTIFICATION;
      }

      $hist->status = NULL;
      $hist->detail = $this->name;
      $hist->simulation = $this->config->simulation;
      $history[] = $hist;
    }

    return EuroSmsHistory::sendMany($history);
  }

  public function save($nullValues = false, $autodate = true) {
    $text = $this->text;
    $sender = $this->sender;
    $unicode = $this->unicode;
    $this->text = static ::encodeLanguages($this->text);
    $this->sender = static ::encodeLanguages($this->sender);
    $this->unicode = static ::encodeLanguages($this->unicode);

    $result = parent::save($nullValues, $autodate);

    $this->text = $text;
    $this->sender = $sender;
    $this->unicode = $unicode;

    return $result;
  }

  public static function transliterate($string) {
    $locale = setlocale(LC_ALL, 0);
    setlocale(LC_ALL, "en_US.utf8");
    $translit = iconv('UTF-8', 'ASCII//TRANSLIT//IGNORE', $string);
    setlocale(LC_ALL, $locale);
    return $translit;
  }

  protected static function replaceTokens($string, $params = NULL, $tokens = NULL) {
    if (!$tokens) {
      $tokens = static ::getTokens();
    }

    $shops = Db::getInstance()->ExecuteS('SELECT `name`, `value` FROM `' . _DB_PREFIX_ . 'configuration` WHERE name like \'PS_SHOP%\' AND (id_shop IS NULL OR id_shop=' . (int)Context::getContext()->shop->id . ') ORDER BY ISNULL(id_shop)');
    $shop = array();
    foreach ($shops as $shop_entry) {
      $name = strtolower(str_replace('PS_SHOP_', '', $shop_entry['name']));
      $shop[$name] = $shop_entry['value'];
    }

    foreach ($tokens as $token => & $info) {
      $token_split = explode('_', $token);

      $prefix = array_shift($token_split);
      $property = array_pop($token_split);
      $container = implode('_', $token_split);

      if ($prefix == 'shop') {
        if (isset($shop[$property])) {
          $info['text'] = $shop[$property];
        }

        continue;
      }

      $object = @$params[$prefix];
      if ($object) {

        if (($prefix == 'order') && ($property == 'total')) {
          $info['text'] = rtrim($object->total_paid, '0');
          continue;
        }

        if (($prefix == 'order') && ($property == 'tracking')) {
          $info['text'] = $object->shipping_number;
          continue;
        }

        if (($prefix == 'customer') && ($property == 'phone')) {
          $info['text'] = EuroSmsRecipientFilter::getRecipientPhone($object);
          continue;
        }

        if (isset($object->$property)) {
          $info['text'] = $object->$property;
          continue;
        }

        if ($container == 'billing') {
          $container = new Address($object->id_address_invoice);
        }
        else if ($container == 'shipping') {
          $container = new Address($object->id_address_delivery);
        }

        if (isset($container->$property)) {
          $info['text'] = $container->$property;
          continue;
        }

        if ($property == 'address') {
          $info['text'] = trim($container->address1 . ' ' . $container->address2);
          continue;
        }

        if ($property == 'country') {
          $info['text'] = Country::getNameById(Context::getContext()->language->id, $container->id_country);
          continue;
        }
      }
      else {
        //$info['text'] = self::DEFAULT_TOKEN_VALUE;
        
      }
    }

    unset($info);
    foreach ($tokens as $token => $info) {
      $string = str_replace($info['pattern'], $info['text'], $string);
    }

    return $string;
  }

  public static function getTokens($allowed = NULL) {
    $result = array();
    foreach (static ::$tokens as $token) {
      $text_split = explode('_', $token);
      $prefix = array_shift($text_split);
      if (isset($allowed) && !in_array($prefix, $allowed)) {
        continue;
      }
      $result[$token] = array(
        'text' => strtoupper(end($text_split)) ,
        'pattern' => '{' . $token . '}',
        'jsPattern' => '\\{' . $token . '\\}'
      );
    }

    ksort($result);
    return $result;
  }

  public static function getNotifications($prefix = self::DEFAULT_PREFIX) {
    $sql = 'SELECT n.* FROM ' . _DB_PREFIX_ . 'euro_sms_notification n';
    if ($prefix) {
      $sql .= ' WHERE name LIKE \'' . pSQL($prefix) . '%\'';
    }

    $result = Db::getInstance()->executeS($sql);
    $notifications = array();
    foreach ($result as & $entry) {
      if ($prefix) {
        $entry['name'] = substr($entry['name'], strlen($prefix));
      }

      $entry['sender'] = static ::decodeLanguages($entry['sender']);
      $entry['text'] = static ::decodeLanguages($entry['text']);
      $entry['unicode'] = static ::decodeLanguages($entry['unicode']);
      $notifications[$entry['name']] = $entry;
    }

    return $notifications;
  }

  public static function getNotificationByName($name) {
    $result = Db::getInstance()->executeS('SELECT n.id_euro_sms_notification as id FROM ' . _DB_PREFIX_ . 'euro_sms_notification n WHERE name = \'' . pSQL($name) . '\'');
    if ($result) {
      return new EuroSmsNotification($result[0]['id']);
    }
    else {
      return new EuroSmsNotification();
    }
  }
}