<?php
require_once (__DIR__ . '/../EuroSmsModel.php');
require_once (_PS_MODULE_DIR_ . 'eurosms/classes/3rdparty/libphonenumber/autoload.php');

class EuroSmsRecipientFilter extends EuroSmsModel {
  public $id_euro_sms_recipient_filter;
  public $name = '';
  public $filter = '';

  public static $definition = array(
    'table' => 'euro_sms_recipient_filter',
    'primary' => 'id_euro_sms_recipient_filter',

    //'multilang' => true,
    'multilang_shop' => true,
    'multishop' => true,

    'associations' => array() ,
    'fields' => array(
      'id_euro_sms_recipient_filter' => array(
        'type' => self::TYPE_INT,
        'validate' => 'isUnsignedId',
        'copy_post' => false
      ) ,
      'name' => array(
        'type' => self::TYPE_STRING,
        'required' => true
      ) ,
      'filter' => array(
        'type' => self::TYPE_STRING,
        'required' => true
      ) ,
    ) ,
  );

  function __construct($id = NULL) {
    parent::__construct($id);
    $this->filter = @json_decode($this->filter);
  }

  public static function getRecipientByPhone($phone) {
    $recips = static ::getRecipients();
    foreach ($recips as $recip) {
      if ($recip['phone'] == $phone) {
        return $recip;
      }
    }

    return false;
  }

  public static function getRecipientPhone($recip) {
    $addrs = $recip->getAddresses(Context::getContext()->language->id);
    foreach($addrs as $addr) {
      $phone = trim($addr['phone_mobile']);
      if (!$phone) {
        $phone = trim($addr['phone']);
      }

      if ($phone) {
        $country = new Country($addr['id_country']);
        $phone = static ::formatPhoneNumber($phone, $country->iso_code);
        return $phone;
      }
    }

    return false;
  }

  public static function getRecipients($filter = array()) {
    static $cache = array();
    $hash = sha1(serialize($filter));

    if (!isset($cache[$hash])) {
      $sql = 'SELECT `id_customer` FROM `' . _DB_PREFIX_ . 'customer` c';
      $sql .= ' ';

      $groups = array();
      foreach ($filter as $filterEntry) {
        $groups[$filterEntry['filter']][] = array(
          'operator' => $filterEntry['operator'],
          'value' => $filterEntry['value'],
          'value2' => $filterEntry['value2']
        );
      }

      //var_dump($groups); die();
      $joins = array();
      foreach ($groups as $which => $group) {
        if ($which) {
          switch ($which) {
            case 'city':
            case 'country':
              $joins[] = 'INNER JOIN ' . _DB_PREFIX_ . 'address a USING (id_customer)';
            break;

            case 'orderSum':
              $joins[] = 'INNER JOIN ' . _DB_PREFIX_ . 'orders o USING (id_customer)';
            break;

            case 'product':
              $joins[] = 'INNER JOIN ' . _DB_PREFIX_ . 'orders o USING (id_customer) INNER JOIN ' . _DB_PREFIX_ . 'order_detail od USING(id_order)';
            break;

            case 'group':
              $joins[] = 'INNER JOIN ' . _DB_PREFIX_ . 'customer_group cg USING (id_customer)';
            break;
          }
        }
      }

      $sql .= implode(' ', array_unique($joins));
      $sql .= ' ';
      $sql .= 'WHERE c.active AND NOT c.deleted';
      $sql .= ' ';

      $andsql = [];
      foreach ($groups as $which => $group) {
        $orsql = [];
        foreach ($group as $filterEntry) {
          if ($which) {
            switch ($which) {
              case 'firstName':
                $orsql[] = 'c.firstname LIKE \'' . pSQL($filterEntry['value']) . '\'';
              break;

              case 'lastName':
                $orsql[] = 'c.lastname = \'' . pSQL($filterEntry['value']) . '\'';
              break;

              case 'gender':
                $orsql[] = 'c.id_gender = ' . (int)$filterEntry['value'];
              break;

              case 'newsletter':
                $orsql[] = 'c.newsletter = ' . (int)$filterEntry['value'];
              break;

              case 'registerDate':
                if ($filterEntry['operator'] == '<=>') {
                  $orsql[] = 'c.date_add >= \'' . pSQL($filterEntry['value']) . '\' AND c.date_add <= \'' . pSQL($filterEntry['value2']) . '\'';
                }
                else {
                  $orsql[] = 'c.date_add ' . pSQL($filterEntry['operator']) . ' \'' . pSQL($filterEntry['value']) . '\'';
                }
              break;

              case 'orderSum':
                $orsql[] = 'o.total_paid ' . pSQL($filterEntry['operator']) . ' \'' . pSQL($filterEntry['value']) . '\'';
              break;

              case 'shop':
                $orsql[] = 'c.id_shop = ' . (int)$filterEntry['value'];
              break;

              case 'city':
                $orsql[] = 'a.city LIKE \'' . pSQL($filterEntry['value']) . '\'';
              break;

              case 'country':
                $orsql[] = 'a.id_country =' . (int)$filterEntry['value'];
              break;

              case 'group':
                $orsql[] = 'cg.id_group = ' . (int)$filterEntry['value'];
              break;

              case 'product':
                $orsql[] = 'od.product_id = ' . (int)$filterEntry['value'];
              break;

              case 'birthDate':
                $md = date('n-j', strtotime((string)$filterEntry['value']));
                $md2 = date('n-j', strtotime((string)@$filterEntry['value2']));
                if ($filterEntry['operator'] == '<=>') {
                  $orsql[] = 'CONCAT(MONTH(c.birthday), \'-\', DAY(c.birthday)) >= \'' . $md . '\' AND CONCAT(MONTH(c.birthday), \'-\', DAY(c.birthday)) <= \'' . $md2 . '\'';
                }
                else {
                  $orsql[] = 'CONCAT(MONTH(c.birthday), \'-\', DAY(c.birthday)) ' . pSQL($filterEntry['operator']) . ' \'' . $md . '\'';
                }
              break;
            }
          }
        }

        $andsql[] = '(' . implode(' OR ', $orsql) . ')';
      }

      if ($andsql) {
        $sql .= 'AND ' . implode(' AND ', $andsql);
      }

      $sql .= ' ';
      $sql .= 'GROUP BY c.`id_customer`';

      $sql .= ' ';
      $sql .= 'ORDER BY c.`id_customer` ASC';


      //var_dump($sql);
      $db = Db::getInstance();
      $recips = $db->ExecuteS($sql);
      if ($recips === false) {
        return $db->getMsgError();
      }

      $recipients = array();

      $phones = array();
      foreach ($recips as $recip) {
        $recipient = $recip;
        $id = $recip['id_customer'];
        $recip_obj = new Customer($id);
        $phone = static ::getRecipientPhone($recip_obj);
        if ($phone && !isset($phones[$phone])) {
          $phones[$phone] = true;
          $recipient['firstname'] = $recip_obj->firstname;
          $recipient['lastname'] = $recip_obj->lastname;
          $recipient['id'] = $recip['id_customer'];
          $recipient['phone'] = $phone;
          $recipients[$recip['id_customer']] = $recipient;
        }
      }
  
      $cache[$hash] = $recipients;
    }

    return $cache[$hash];
  }

  public function getFilteredRecipients() {
    $filter = $this->filter;
    return static ::getRecipients($filter);
  }

  public static function formatPhoneNumber($number, $country = 'SK') {
    $phoneUtil = \libphonenumber\PhoneNumberUtil::getInstance();
    try {
      $number = trim($number);
      $number = str_replace(' ', '', $number);
      $phone_parsed = $phoneUtil->parse($number, $country);
      $phone_formatted = str_replace(' ', '', $phoneUtil->format($phone_parsed, \libphonenumber\PhoneNumberFormat::INTERNATIONAL));
    }
    catch(\libphonenumber\NumberParseException $e) {
      $phone_formatted = $number;
    }

    return $phone_formatted;
  }

  public static function getFilters() {
    $result = Db::getInstance()->executeS('SELECT f.* FROM ' . _DB_PREFIX_ . 'euro_sms_recipient_filter f ORDER BY name ASC');
    $filters = array();
    foreach ($result as $entry) {
      $filter = @json_decode($entry['filter']);
      if (!$filter) {
        $filter = new stdClass();
      }
      $filter->id_euro_sms_recipient_filter = $entry['id_euro_sms_recipient_filter'];
      $filter->filter = $entry['filter'];
      $filters[] = array(
        'name' => $entry['name'],
        'data' => $filter
      );
    }

    return $filters;
  }
}
