<?php
require_once ('EuroSmsNotification.php');
require_once ('EuroSmsConfiguration.php');
class EuroSmsSending extends EuroSmsModel {

  public $id_euro_sms_sending = null;
  public $notification_name = '';
  public $recipients = '';
  public $time = null;
  public $template = false;
  public $id_shop = null;

  public $notification = null;
  public $type = null;

  public static $definition = array(
    'table' => 'euro_sms_sending',
    'primary' => 'id_euro_sms_sending',

    //'multilang' => true,
    'multilang_shop' => true,
    'multishop' => true,

    'associations' => array(
      'shop' => array(
        'type' => self::HAS_ONE,
        'field' => 'id_shop',
        'object' => 'Shop'
      ) ,
      'euro_sms_notification' => array(
        'type' => self::HAS_ONE,
        'field' => 'notification_name',
        'object' => 'EuroSmsNotification'
      )
    ) ,
    'fields' => array(
      'id_euro_sms_sending' => array(
        'type' => self::TYPE_INT,
        'validate' => 'isUnsignedId',
        'copy_post' => false
      ) ,
      'notification_name' => array(
        'type' => self::TYPE_STRING,
        'required' => true
      ) ,
      'recipients' => array(
        'type' => self::TYPE_STRING,
        'required' => false,
      ) ,
      'time' => array(
        'type' => self::TYPE_DATE,
        'validate' => 'isDate',
        'required' => false
      ) ,
      'template' => array(
        'type' => self::TYPE_STRING,
        'required' => false
      ) ,
      'id_shop' => array(
        'type' => self::TYPE_INT,
        'validate' => 'isNullOrUnsignedId',
        'required' => false,
        'allow_null' => true
      ) ,
    )
  );

  public static function encodeTime($value) {
    if ($value) {
      $time = strtotime(trim($value['date'] . ' ' . $value['time']));
      $time = date('Y-m-d H:i:s', $time);
    }
    else {
      $time = $value;
    }

    return $time;
  }

  public static function decodeTime($value) {
    if (strtotime($value)) {
      $time_split = explode(' ', $value);
      $time = array(
        'time' => $time_split[1],
        'date' => $time_split[0]
      );
    }
    else {
      $time = NULL;
    }

    return $time;
  }

  function __construct($id = NULL) {
    parent::__construct($id);
    $this->time = static::decodeTime($this->time);
  }

  public function save($nullValues = false, $autodate = true) {

    $this->time = static::encodeTime($this->time);
    if ($this->template) {
      $this->recipients = null;
    }

    if ($this->notification && $this->type) {
      $this->notification->active = !$this->template;
      $this->notification->name = uniqid($this->type . '.', true);
      if (!$this->notification->esSave()) {
        return false;
      }

      $this->notification_name = $this->notification->name;
    }

    $result = parent::save($nullValues, $autodate);

    if ($this->notification && $this->notification->active) {
      $recipients = array_unique(explode(' ', $this->recipients));

      $params = array();
      foreach($recipients as $key => $recipient) {
        $recip = EuroSmsRecipientFilter::getRecipientByPhone($recipient);
        if ($recip) {
          $params[$key]['customer'] = new Customer($recip['id_customer']);
        }
      }

      $this->notification->send($recipients, array(
        'time' => $this->time,
        'type' => $this->type,
        'params' => $params
      ));
    }

    return $result;
  }

  public static function getTemplates() {
    $result = Db::getInstance()->executeS('SELECT s.*, n.* FROM ' . _DB_PREFIX_ . 'euro_sms_sending s INNER JOIN ' . _DB_PREFIX_ . 'euro_sms_notification n ON (n.name = s.notification_name) WHERE template IS NOT NULL ORDER BY template ASC');
    $templates = array();
    foreach ($result as $entry) {
      $entry['sender'] = EuroSmsNotification::decodeLanguages($entry['sender']);
      $entry['text'] = EuroSmsNotification::decodeLanguages($entry['text']);
      $entry['unicode'] = EuroSmsNotification::decodeLanguages($entry['unicode']);
      $entry['time'] = static::decodeTime($entry['time']);
      $templates[] = array(
        'template' => $entry['template'],
        'data' => $entry
      );
    }

    return $templates;
  }
}
