<?php
require_once ('AdminEuroSmsSendingController.php');
class AdminEuroSmsMarketingController extends AdminEuroSmsSendingController {

  function __construct() {
    parent::__construct();

    require_once (_PS_MODULE_DIR_ . 'eurosms/classes/model/EuroSmsMarketing.php');
    $this->className = 'EuroSmsMarketing';
    $this->meta_title = $this->l('SMS Marketing');
    $this->toolbar_title = $this->l('SMS Marketing');
  }

  public function renderView() {
    $gndrs = Gender::getGenders();
    $genders = array();
    foreach ($gndrs as $gndr) {
      $genders[$gndr->id_gender] = array(
        'id_gender' => $gndr->id_gender,
        'id' => $gndr->id_gender,
        'name' => $gndr->name
      );
    }

    $this->tpl_view_vars['genders'] = $genders;

    $grps = Group::getGroups(Context::getContext()->language->id);
    $groups = array();
    foreach ($grps as $grp) {
      if (!$grp['id_group']) {
        continue;
      }
      $groups[$grp['id_group']] = array(
        'id_group' => $grp['id_group'],
        'id' => $grp['id_group'],
        'name' => $grp['name']
      );
    }

    $this->tpl_view_vars['groups'] = $groups;

    $yesno = array(
      0 => array(
        'id' => 0,
        'name' => $this->l('No')
      ) ,
      1 => array(
        'id' => 1,
        'name' => $this->l('Yes')
      )
    );
    $this->tpl_view_vars['yesno'] = $yesno;

    $prods = Product::getProducts(Context::getContext()->language->id, 0, 0, 'name', 'ASC');
    $products = array();
    foreach ($prods as $prod) {
      if (!$prod['id_product']) {
        continue;
      }
      $products[$prod['id_product']] = array(
        'id_product' => $prod['id_product'],
        'id' => $prod['id_product'],
        'name' => $prod['name'] . ' (' . $prod['reference'] . ')'
      );
    }

    $this->tpl_view_vars['products'] = $products;

    $all_recipients = EuroSmsRecipientFilter::getRecipients();
    $this->tpl_view_vars['allRecipients'] = $all_recipients;

    return parent::renderView();
  }

  protected function saveData($marketing, $type = EuroSmsHistory::TYPE_MARKETING) {
    return parent::saveData($marketing, $type);
  }

  protected function esProcessAjax($ajaxCmd) {
    switch ($ajaxCmd) {
      case 'filterLoad':
        $filters = EuroSmsRecipientFilter::getFilters();
        return $filters;
      break;

      case 'filterSave':
        $filter = new EuroSmsRecipientFilter();
        $filter->esSetFromRequest();
        $return = $filter->esSave();
        if ($return) {
          return 0;
        }
        else {
          return html_entity_decode(reset($this->errors));
        }
      break;

      case 'filterDelete':
        $id = Tools::getValue('id'); 
        if ($id) {
          $filter = new EuroSmsRecipientFilter($id);
          if ($this->esDeleteModel($filter, $this->l('Filter deleted.'))) {
            return 0;
          }
          else {
            return html_entity_decode(reset($this->errors));
          }
        }
      break;

      case 'filterApply':
        $filter = Tools::getValue('filter');
        $filter = @json_decode($filter, true);
        if ($filter !== FALSE) {
          $recipients = EuroSmsRecipientFilter::getRecipients($filter);
          return $recipients;
        }
      break;

      default:
        return parent::esProcessAjax($ajaxCmd);
    }
  }
}
