<?php
require_once (_PS_MODULE_DIR_ . 'eurosms/classes/AdminEuroSmsController.php');
require_once (_PS_MODULE_DIR_ . '/eurosms/classes/model/EuroSmsConfiguration.php');
require_once (_PS_MODULE_DIR_ . '/eurosms/classes/model/EuroSmsNotification.php');
require_once (_PS_MODULE_DIR_ . '/eurosms/classes/model/EuroSmsRecipientFilter.php');
class AdminEuroSmsNotificationsController extends AdminEuroSmsController {

  const CART_NOTIFY_INTERVAL = 86400;
  //const CART_NOTIFY_INTERVAL = 60;

  public static $notificationStates = array(
    'order' => array(
      'cart' => array(
        'title' => 'Order unfinished (in cart)',
        'hook' => 'cron',
        'tokens' => array(
          'customer',
          'shop'
        )
      ) ,
      'new' => array(
        'title' => 'Order received',
        'hook' => 'actionValidateOrder'
      ) ,
      'paid' => array(
        'title' => 'Order paid',
        'hook' => 'actionOrderStatusPostUpdate',
        'id' => array(
          2,
          9,
          12
        )
      ) ,
      'preparation' => array(
        'title' => 'Order preparation',
        'hook' => 'actionOrderStatusPostUpdate',
        'id' => array(
          3
        )
      ) ,
      'shipped' => array(
        'title' => 'Order shipped',
        'hook' => 'actionOrderStatusPostUpdate',
        'id' => array(
          4
        )
      ) ,
      'tracking' => array(
        'title' => 'Order tracking number',
        'hook' => 'actionOrderStatusPostUpdate',
        'id' => array(
          4
        )
      ) ,
    ) ,

    'admin' => array(
      'order.new' => array(
        'title' => 'Order received',
        'hook' => 'actionValidateOrder'
      ) ,
    )
  );

  public static function notificationStateOrderCart() {

    $now = time();
    $now_str = date('Y-m-d H:i:s', $now);
    $dateTo = $now - static ::CART_NOTIFY_INTERVAL;
    $dateFrom = $now - 2 * static ::CART_NOTIFY_INTERVAL;

    $sql = 'SELECT cart.`id_cart`, cart.`date_upd`, c.`id_customer` AS id_customer, c.`lastname` AS customer_lastname, c.`firstname` AS customer_firstname,
    SUM(cp.`quantity`) AS nb_products
    FROM `' . _DB_PREFIX_ . 'cart` cart
    LEFT JOIN `' . _DB_PREFIX_ . 'cart_product` cp ON cart.`id_cart` = cp.`id_cart`
    LEFT JOIN `' . _DB_PREFIX_ . 'customer` c ON cart.`id_customer` = c.`id_customer`
    WHERE cart.`id_cart` NOT IN (SELECT `id_cart` FROM `' . _DB_PREFIX_ . 'orders`)
    AND cart.`date_upd` >= \'' . pSQL(strftime('%Y-%m-%d %H:%M:%S', $dateFrom)) . '\'
    AND cart.`date_upd` <= \'' . pSQL(strftime('%Y-%m-%d %H:%M:%S', $dateTo)) . '\'
    GROUP BY cart.`id_cart`, cp.`id_cart`
    ORDER BY cart.`date_upd` DESC';

    $carts = Db::getInstance()->ExecuteS($sql);

    $recips = array();
    $params = array();

    foreach ($carts as $cart) {
      $cart_obj = new Cart($cart['id_cart']);
      if (!$cart_obj->id_customer) {
        continue;
      }

      $customer = new Customer($cart_obj->id_customer);
      $recip = EuroSmsRecipientFilter::getRecipientPhone($customer);
      if (!$recip) {
        continue;
      }

      $params[] = array(
        'customer' => $customer
      );

      $recips[] = $recip;

      $cart_obj->date_upd = $now_str;
      $cart_obj->save();
    }

    if ($recips) {
      $notif = EuroSmsNotification::loadByName('@order.cart', true);
      if (!$notif) {
        return;
      }

      $notif->send($recips, array('params' => $params));
    }
  }

  public static function notificationStateOrderNew($cart, $order, $customer, $currency, $orderStatus) {
    $recip = EuroSmsRecipientFilter::getRecipientPhone($customer);
    if (!$recip) {
      return;
    }

    $notif = EuroSmsNotification::loadByName('@order.new', true);
    if (!$notif) {
      return;
    }

    $notif->send(array(
      $recip
    ) , array(
      'params' => array(
        array(
          'customer' => $customer,
          'order' => $order
        )
      )
    ));
  }

  public static function notificationStateOrderPaid($new_status, $id_order) {
    if (in_array($new_status->id, static ::$notificationStates['order']['paid']['id'])) {
      $order = new Order($id_order);
      $customer = new Customer($order->id_customer);

      $recip = EuroSmsRecipientFilter::getRecipientPhone($customer);
      if (!$recip) {
        return;
      }

      $notif = EuroSmsNotification::loadByName('@order.paid', true);
      if (!$notif) {
        return;
      }

      $notif->send(array(
        $recip
      ) , array(
        'params' => array(
          array(
            'customer' => $customer,
            'order' => $order
          )
        )
      ));
    }
  }

  public static function notificationStateOrderPreparation($new_status, $id_order) {
    if (in_array($new_status->id, static ::$notificationStates['order']['preparation']['id'])) {
      $order = new Order($id_order);
      $customer = new Customer($order->id_customer);

      $recip = EuroSmsRecipientFilter::getRecipientPhone($customer);
      if (!$recip) {
        return;
      }

      $notif = EuroSmsNotification::loadByName('@order.preparation', true);
      if (!$notif) {
        return;
      }

      $notif->send(array(
        $recip
      ) , array(
        'params' => array(
          array(
            'customer' => $customer,
            'order' => $order
          )
        )
      ));
    }
  }

  public static function notificationStateOrderShipped($new_status, $id_order) {
    if (in_array($new_status->id, static ::$notificationStates['order']['shipped']['id'])) {
      $order = new Order($id_order);

      if (trim($order->shipping_number)) {
        return;
      }

      $customer = new Customer($order->id_customer);

      $recip = EuroSmsRecipientFilter::getRecipientPhone($customer);
      if (!$recip) {
        return;
      }

      $notif = EuroSmsNotification::loadByName('@order.shipped', true);
      if (!$notif) {
        return;
      }

      $notif->send(array(
        $recip
      ) , array(
        'params' => array(
          array(
            'customer' => $customer,
            'order' => $order
          )
        )
      ));
    }
  }

  public static function notificationStateOrderTracking($new_status, $id_order) {
    if (in_array($new_status->id, static ::$notificationStates['order']['tracking']['id'])) {
      $order = new Order($id_order);

      if (!trim($order->shipping_number)) {
        return;
      }

      $customer = new Customer($order->id_customer);

      $recip = EuroSmsRecipientFilter::getRecipientPhone($customer);
      if (!$recip) {
        return;
      }

      $notif = EuroSmsNotification::loadByName('@order.tracking', true);
      if (!$notif) {
        return;
      }

      $notif->send(array(
        $recip
      ) , array(
        'params' => array(
          array(
            'customer' => $customer,
            'order' => $order
          )
        )
      ));
    }
  }

  public static function notificationStateAdminOrderNew($cart, $order, $customer, $currency, $orderStatus) {
    $config = new EuroSmsConfiguration();
    $admins = $config->getAdministrators();

    $recips = array();
    foreach ($admins as $admin) {
      $recips[] = $admin->phone;
    }

    if (!$recips) {
      return;
    }

    $notif = EuroSmsNotification::loadByName('@admin.order.new', true);
    if (!$notif) {
      return;
    }

    $notif->send($recips, array(
      'params' => array(
        array(
          'customer' => $customer,
          'order' => $order
        )
      )
    ));
  }

  function __construct() {
    parent::__construct();

    $this->className = 'EuroSmsNotification';
    $this->meta_title = $this->l('SMS Notifications');
    $this->toolbar_title = $this->l('SMS Notifications');

    global $eurosms;
    foreach (static ::$notificationStates as $group => $states) {
      foreach ($states as $name => $state) {
        $nameSplit = explode('.', $name);
        $fncName = 'notificationState' . ucfirst($group) . implode('', array_map('ucfirst', $nameSplit));
        if ($state['hook'] == 'cron') {
          $eurosms->registerCronHandler(__FILE__, 'AdminEuroSmsNotificationsController', $fncName);
        }
        else {
          $eurosms->registerHookHandler($state['hook'], __FILE__, 'AdminEuroSmsNotificationsController', $fncName);
        }
      }
    }
  }

  public function renderView() {
    $notificationStates = static ::$notificationStates;
    foreach ($notificationStates as & $group) {
      foreach ($group as & $notificationState) {
        if (isset($notificationState['tokens'])) {
          $notificationState['tokens'] = EuroSmsNotification::getTokens($notificationState['tokens']);
        }
        else {
          $notificationState['tokens'] = EuroSmsNotification::getTokens();
        }
      }
    }

    unset($group);
    unset($notificationState);

    $this->tpl_view_vars['notificationStates'] = $notificationStates;

    $notifications = EuroSmsNotification::getNotifications();

    $empty = array();
    $cnt = 0;
    foreach (Language::getLanguages(true) as $lang) {
      $cnt++;
      $empty[$lang['iso_code']] = '';
      if ($cnt >= EUROSMS_MAX_LANGUAGES) {
        break;
      }
    }

    foreach ($notificationStates as $group => $states) {
      foreach ($states as $key => $state) {
        $name = $group . '.' . $key;
        if (!isset($notifications[$name])) {
          $notifications[$name] = array(
            'active' => true,
            'sender' => $empty,
            'text' => $empty,
            'unicode' => false
          );
        }
      }
    }

    $this->tpl_view_vars['notifications'] = $notifications;
    return parent::renderView();
  }

  public function postProcess() {
    $save = Tools::isSubmit('save');

    if ($save) {
      $done = true;
      $notificationsData = Tools::getValue('notification');
      foreach ($notificationsData as $notificationName => $notificationData) {
        $notificationKey = EuroSmsNotification::DEFAULT_PREFIX . $notificationName;
        $notification = EuroSmsNotification::getNotificationByName($notificationKey);
        $notification->esSet($notificationData);
        $notification->name = $notificationKey;
        $done = $done && $notification->esSave();
      }

      if ($done) {
        $this->confirmations = $this->l('Notifications saved.');
      }
    }
  }

  protected function esProcessAjax($ajaxCmd) {
    switch ($ajaxCmd) {

      case 'notificationSave':
        $name = Tools::getValue('name');
        $notificationKey = EuroSmsNotification::DEFAULT_PREFIX . $name;
        $notification = EuroSmsNotification::getNotificationByName($notificationKey);
        $notification->esSetFromRequest();
        $notification->name = $notificationKey;
        $return = $notification->esSave();
        if ($return) {
          return 0;
        }
        else {
          return html_entity_decode(reset($this->errors));
        }
      break;

      default:
        return parent::esProcessAjax($ajaxCmd);
    }
  }
}