<?php 
/**
 *
 * @author    Pexxi:solutions <support@pexxi.eu>
 * @copyright 2020 Pexxi:solutions
 * @license   Commercial
 * @version   1.6.0
 */

if (!defined('_PS_VERSION_')) exit;

define('EUROSMS_DEBUG', FALSE);
/*if (EUROSMS_DEBUG) {
  error_reporting(E_ALL & ~E_NOTICE & ~E_DEPRECATED);
  ini_set('display_errors', 1);
  ini_set('display_startup_errors', 1);
}*/

class eurosms extends Module {

  /**
   * __construct function.
   *
   * @access public
   * @return void
   */
  public function __construct() {
    global $eurosms;
    $eurosms = $this;

    $this->name = 'eurosms';
    $this->module_key = '3cf88d914ce04c0d2c19a3a4fd90e162';
    $this->version = '1.0.4';
    $this->tab = 'emailing';
    $this->need_instance = 0;

    parent::__construct();

    $this->displayName = $this->l('EuroSMS – SMS and Viber messaging');
    $this->description = $this->l('Send Notifications, Marketing, Bulk and Transactional SMS/Viber messages via Bank Level Connections to mobile operators across the Globe for very competitive prices in each country.');
    $this->author = 'Pexxi:solutions';

    if (Tools::getValue('esTest')) {
      $this->esTestHook();
    }
  }

  /**
   * install function.
   *
   * @access public
   * @return void
   */
  public function install() {
    $sqlHistory = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . "euro_sms_history` (
                  `id_euro_sms_history` INT(10) UNSIGNED NOT NULL AUTO_INCREMENT,
                  `sender` VARCHAR(128),
                  `recipient` VARCHAR(128) NOT NULL,
                  `timestamp` TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
                  `delivery_timestamp` TIMESTAMP,
                  `type` ENUM('sending', 'marketing', 'notification') NOT NULL,
                  `simulation` BOOL NOT NULL DEFAULT 0,
                  `status` ENUM('pending', 'sent', 'delivered', 'read', 'undelivered') NOT NULL,
                  `error` TEXT,
                  `detail` TEXT,
                  `text` TEXT NOT NULL,
                  `price` DECIMAL(10, 2) NOT NULL DEFAULT 0,
                  `unicode` VARCHAR(128) NOT NULL,
                  `sms_id` VARCHAR(864),
                  `group_id` VARCHAR(128),
                  `sms_count` INT NOT NULL DEFAULT 1,
                  PRIMARY KEY (`id_euro_sms_history`),
                  INDEX sender (`sender`),
                  INDEX recipient (`recipient`),
                  INDEX `timestamp` (`timestamp`),
                  INDEX `delivery_timestamp` (`delivery_timestamp`),
                  INDEX `type` (`type`),
                  INDEX simulation (`simulation`),
                  INDEX group_id (`group_id`),
                  UNIQUE INDEX sms_id (`sms_id`(128))
                ) ENGINE=" . _MYSQL_ENGINE_ . " DEFAULT CHARSET=utf8";
    $sqlNotification = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . "euro_sms_notification` (
                  `id_euro_sms_notification` int(10) UNSIGNED NOT NULL AUTO_INCREMENT,
                  `name` VARCHAR(128) NOT NULL,
                  `sender` VARCHAR(128),
                  `text` TEXT,
                  `active` BOOL NOT NULL DEFAULT 0,
                  `unicode` VARCHAR(128),
                  PRIMARY KEY (`id_euro_sms_notification`),
                  UNIQUE INDEX `name` (`name`),
                  INDEX active (`active`)
                ) ENGINE=" . _MYSQL_ENGINE_ . " DEFAULT CHARSET=utf8";
    $sqlConfiguration = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . "euro_sms_configuration` (
                  `id_euro_sms_configuration` int(10) unsigned NOT NULL AUTO_INCREMENT,
                  `email` VARCHAR(256) NOT NULL,
                  `personal` BOOL NOT NULL DEFAULT 0,
                  `company_name` VARCHAR(128) NOT NULL,
                  `company_id` VARCHAR(32),
                  `company_vat` VARCHAR(32),
                  `contact_first_name` VARCHAR(128),
                  `contact_last_name` VARCHAR(128),
                  `address_street` VARCHAR(128),
                  `address_city` VARCHAR(128),
                  `address_zip` VARCHAR(32),
                  `address_id_country` INT(10) UNSIGNED,
                  `user_name` VARCHAR(128),
                  `user_password` VARCHAR(128),
                  `administrators` TEXT,
                  `phone_invoice` BOOL NOT NULL DEFAULT 0,
                  `credit_alert` VARCHAR(256),
                  `cron_key` VARCHAR(128) NOT NULL,
                  `simulation` BOOL NOT NULL DEFAULT 1,
                  `delivery_reports` BOOL NOT NULL DEFAULT 1,
                  PRIMARY KEY (`id_euro_sms_configuration`),
                  FOREIGN KEY (address_id_country) REFERENCES " . _DB_PREFIX_ . "country(id_country) ON DELETE SET NULL
                ) ENGINE=" . _MYSQL_ENGINE_ . " DEFAULT CHARSET=utf8";
    $sqlRecipientFilter = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . "euro_sms_recipient_filter` (
                  `id_euro_sms_recipient_filter` int(10) unsigned NOT NULL AUTO_INCREMENT,
                  `name` VARCHAR(128) NOT NULL,
                  `filter` TEXT NOT NULL,
                  PRIMARY KEY (`id_euro_sms_recipient_filter`),
                  UNIQUE INDEX `name` (`name`)
                ) ENGINE=" . _MYSQL_ENGINE_ . " DEFAULT CHARSET=utf8";
    $sqlSending = 'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . "euro_sms_sending` (
                  `id_euro_sms_sending` int(10) UNSIGNED NOT NULL AUTO_INCREMENT,
                  `notification_name` VARCHAR(128) NOT NULL,
                  `recipients` TEXT,
                  `time` TIMESTAMP,
                  `template` VARCHAR(128),
                  `id_shop` INT(10) UNSIGNED,
                  PRIMARY KEY (`id_euro_sms_sending`),
                  INDEX notification_name (`notification_name`),
                  INDEX `time` (`time`),
                  UNIQUE INDEX `template` (`template`),
                  FOREIGN KEY (notification_name) REFERENCES " . _DB_PREFIX_ . "euro_sms_notification(name) ON DELETE CASCADE,
                  FOREIGN KEY (id_shop) REFERENCES " . _DB_PREFIX_ . "shop(id_shop) ON DELETE SET NULL
                ) ENGINE=" . _MYSQL_ENGINE_ . " DEFAULT CHARSET=utf8";

    if (!parent::install() || !$this->registerHook('actionValidateOrder') || !$this->registerHook('actionOrderStatusPostUpdate') || !$this->registerHook('displayBackOfficeHeader') || !Configuration::updateValue('PEXXI_MODULES_LIST', true) || !Configuration::updateValue('PEXXI_MODULE_TIME', 0) || !Db::getInstance()->execute($sqlHistory) || !Db::getInstance()->execute($sqlNotification) || !Db::getInstance()->execute($sqlConfiguration) || !Db::getInstance()->execute($sqlSending) || !Db::getInstance()->execute($sqlRecipientFilter)) {
      print Db::getInstance()->getMsgError();
      die();
      return false;
    }

    $currentId = Tab::getIdFromClassName('AdminEuroSms');
    $subTabs = array(
      'Notifications',
      'Marketing',
      'Sending',
      'Statistics',
      'History',
      'Configuration'
    );

    if (!$currentId) {
      $tab = new Tab();
      $tab->active = 1;
      $tab->class_name = "AdminEuroSms";
      $tab->name = array();
      foreach (Language::getLanguages() as $lang) {
        $tab->name[$lang['id_lang']] = "EuroSMS";
      }

      $tab->id_parent = 0;
      $tab->module = $this->name;
      $tab->add();
      $parentId = Tab::getIdFromClassName('AdminEuroSms');
    }
    else {
      $parentId = $currentId;
    }

    if ($currentId) {
      $subTabs = array();
    }

    foreach ($subTabs as $subTab) {
      $className = "AdminEuroSms" . $subTab;
      /*if (Tab::getIdFromClassName($className)) {
        continue;
      }*/

      $tab = new Tab();
      $tab->active = 1;
      $tab->class_name = $className;
      $tab->name = array();

      foreach (Language::getLanguages() as $lang) {
        $tab->name[$lang['id_lang']] = $subTab;
      }

      $tab->id_parent = $parentId;
      $tab->module = $this->name;
      $tab->add();
    }

    return true;
  }

  /**
   * uninstall function.
   *
   * @access public
   * @return void
   */
  public function uninstall() {
    $currentId = Tab::getIdFromClassName('AdminEuroSms');
    if ($currentId) {
      $tab = new Tab($currentId);
      $tab->delete();
    }

    // NOTE: remove tables?
    if (!parent::uninstall() /*|| !Configuration::deleteByName('PEXXI_MODULE_TIME')*/) {
      return false;
    }

    return true;
  }

  public function hookDisplayBackOfficeHeader($params) {
    $controller = Tools::getValue('controller');
    if (strpos($controller, 'EuroSms') !== false) {
      $this->context->controller->addJquery();

      $this->context->controller->addCSS(($this->_path) . 'views/css/ui.css');
      $this->context->controller->addCSS(($this->_path) . 'views/css/custom.css');
      $this->context->controller->addCSS(($this->_path) . 'views/css/override.css');

      $this->context->controller->addJS(($this->_path) . 'views/js/ui.js');
      $this->context->controller->addJS(($this->_path) . 'views/js/pstagger.js');
      $this->context->controller->addJS(($this->_path) . 'views/js/custom.js');
      $this->context->controller->addJS('https://cdnjs.cloudflare.com/ajax/libs/libphonenumber-js/1.1.9/libphonenumber-js.min.js');
    }

    $this->context->controller->addCSS(($this->_path) . 'views/css/icons.css');
  }

  /**
   * addAsTrusted function.
   *
   * @access public
   * @return void
   */
  public function addAsTrusted() {
    if (defined('static::CACHE_FILE_TRUSTED_MODULES_LIST') == true) {
      if (isset($this->context->controller->controller_name) && $this->context->controller->controller_name == 'AdminModules') {
        $sxe = new SimpleXMLElement('<theme/>');
        $modules = $sxe->addChild('modules');
        $module = $modules->addChild('module');
        $module->addAttribute('action', 'install');
        $module->addAttribute('name', $this->name);
        $trusted = $sxe->saveXML();
        file_put_contents(_PS_ROOT_DIR_ . '/config/xml/themes/' . $this->name . '.xml', $trusted);
        if (is_file(_PS_ROOT_DIR_ . Module::CACHE_FILE_UNTRUSTED_MODULES_LIST)) Tools::deleteFile(_PS_ROOT_DIR_ . Module::CACHE_FILE_UNTRUSTED_MODULES_LIST);
      }
    }
  }

  public function registerCronHandler($file, $className, $method) {
    $handlers = $this->getCronHandlers();
    $save = false;
    if (!isset($handlers[$file][$className][$method])) {
      $save = true;
      $handlers[$file][$className][$method] = true;
    }

    if ($save) {
      $handlers = json_encode($handlers);
      return Configuration::updateValue('EUROSMS_CRON_HANDLERS', $handlers);
    }
    else {
      return false;
    }
  }

  public function getCronHandlers() {
    $handlers = Configuration::get('EUROSMS_CRON_HANDLERS');
    if ($handlers) {
      $handlers = json_decode($handlers, true);
    }
    else {
      $handlers = array();
    }

    return $handlers;
  }

  public function executeCronHandlers() {
    $handlers = $this->getCronHandlers();
    foreach ($handlers as $file => $h1) {
      require_once ($file);
      foreach ($h1 as $class => $h2) {
        foreach ($h2 as $fnc => $h3) {
          $class::$fnc();
        }
      }
    }
  }

  public function registerHookHandler($hook, $file, $className, $method) {
    $handlers = $this->getHookHandlers();
    $save = false;
    if (!isset($handlers[$hook][$file][$className][$method])) {
      $save = true;
      $handlers[$hook][$file][$className][$method] = true;
    }

    if ($save) {
      $handlers = json_encode($handlers);
      return Configuration::updateValue('EUROSMS_HOOK_HANDLERS', $handlers);
    }
    else {
      return false;
    }
  }

  public function getHookHandlers() {
    $handlers = Configuration::get('EUROSMS_HOOK_HANDLERS');
    if ($handlers) {
      $handlers = json_decode($handlers, true);
    }
    else {
      $handlers = array();
    }

    return $handlers;
  }

  public function hookActionOrderStatusPostUpdate($params) {
    $this->executeHookHandlers('actionOrderStatusPostUpdate', $params);
  }

  protected function esTestHook() {
    $id_order = Tools::getValue('id_order');
    if ($id_order) {
      $this->hookActionValidateOrder(
        array(
          null,
          ($order = new Order($id_order)),
          new Customer($order->id_customer),
          null,
          null
        )
      );
    }
  }

  public function hookActionValidateOrder($params) {
    $this->executeHookHandlers('actionValidateOrder', $params);
  }

  public function executeHookHandlers($hook, $args = array()) {
    $handlers = $this->getHookHandlers();
    if (!empty($handlers[$hook])) {
      foreach ($handlers[$hook] as $file => $h1) {
        require_once ($file);
        foreach ($h1 as $class => $h2) {
          foreach ($h2 as $fnc => $h3) {
            call_user_func_array(array(
              $class,
              $fnc
            ) , $args);
          }
        }
      }
    }
  }
}
